-----------------------------------------------------------------------------------
-- delay d from 2 to 17 clocks with ce = '1'
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library UNISIM;
use UNISIM.VComponents.all;

entity delayN is
    generic (N: positive := 2;
	          INIT : bit_vector := x"0000");
    port (
        c: in STD_LOGIC;
        d: in STD_LOGIC;
        ce: in STD_LOGIC;
        q: out STD_LOGIC
    );
end delayN;
architecture delayN_arch of delayN is
signal a: std_logic_vector(4 downto 0);
signal dl: std_logic;
begin
a <= conv_std_logic_vector(N-2,5);
delay: srl16e
    generic map(INIT => INIT)
    port map(
           d => d,
           ce => ce,
           clk => c,
           a0 => a(0),
           a1 => a(1),
           a2 => a(2),
           a3 => a(3),
           q => dl);
process(c)
begin
  if(c'event and c = '1')then
    if(ce = '1')then
      q <= dl;
    end if;
  end if;
end process;
end delayN_arch;
---------------------------------------------------------------------------------------------------
-- DELAY T for N-bit word
---------------------------------------------------------------------------------------------------
-- delay T from 2 to 17 clocks with ce = '1'
-- N => for N-bit word
  
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library UNISIM;
use UNISIM.VComponents.all;

entity delayt_N is
  generic ( T: positive := 2; -- Minimum value!
   	    N: positive := 16);
    port (
        clk: in STD_LOGIC;
        d: in STD_LOGIC_vector(N-1 downto 0);
        ce: in STD_LOGIC;
        q: out STD_LOGIC_vector(N-1 downto 0)
         );
end delayt_N;

architecture delayt_N_arch of delayt_N is
signal a: std_logic_vector(4 downto 0);
signal dl: std_logic_vector(N-1 downto 0);
begin
a <= conv_std_logic_vector(T-2,5);
gt: for i in d'range generate
delay: srl16e
    port map(
           d => d(I),
           ce => ce,
           clk => clk,
           a0 => a(0),
           a1 => a(1),
           a2 => a(2),
           a3 => a(3),
           q => dl(I));
end generate;      
    
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ce = '1')then
      q <= dl;
    end if;
  end if;
end process;
end delayt_N_arch;
----------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;

entity ddr_io is
    port (
        clk: in STD_LOGIC;
        clkn: in STD_LOGIC;
        pad: inout std_logic;
        t: in STD_LOGIC;
        d0: in STD_LOGIC;
        d1: in STD_LOGIC;
        iclk: in STD_LOGIC;
        iclkn: in STD_LOGIC;
        q0: out std_logic; -- rising edge       
        q1: out std_logic  -- falling edge    
    );
end ddr_io;

architecture ddr_io_arch of ddr_io is
signal din, oq, tq: std_logic;
attribute IOB : string;
attribute IOB of q0_reg,q1_reg,tq_reg: label is "TRUE";
begin
ib: ibuf port map(i => pad, o => din);
q0_reg: fd port map(d => din, c=> iclk, q => q0);
q1_reg: fd port map(d => din, c=> iclkn, q => q1);
oq_reg:  fddrrse
    port map(
          d0 => d0,
          d1 => d1,
          c0 => clk,
          c1 => clkn,
          ce => '1',
          r  => '0',
          s  => '0',
          q  => oq);
tq_reg:  fd
    port map(
          d => t,
          c => clk,
          q  => tq);
ob: obuft port map( i => oq, o => pad, t => tq);
end ddr_io_arch;
-------------------------------------------------------------------------------------
-- this is a loadable up counter with set, reset and clock enable inputs 
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;
entity l_cntr is
generic (N: positive := 8; ld_plus_one : boolean := false );
  port (
    d: in std_logic_vector (N-1 downto 0);
-- load = '1' and ec = '1' loads d into q
    load: in std_logic;
-- reset resets the counter to all zeros, reset overrides set and ec
    reset: in std_logic;
-- set will set the counter to all ones
    set: in std_logic;
-- ec = '1' is needed to load or count 
    ec: in std_logic;
-- en_cnt = '1' is needed to count. If load with en_cnt = '1',
-- d + 1 is loaded. 
    en_cnt: in std_logic;
-- clock input
    clk: in std_logic;
-- q is the counter output
    q: out std_logic_vector (N-1 downto 0);
-- carry out
    carry: out std_logic
  );
end l_cntr;

architecture my_arch of l_cntr is
signal	qi,s,dq:	std_logic_vector(N-1 downto 0) := (others => '0');
signal	ci:	std_logic_vector(N downto 0) := (others => '0');
begin
q <= qi;
carry <= ci(N);
ci(0) <= en_cnt or load when ld_plus_one else en_cnt and not load;
G1: for I in q'range generate
  s(I) <= (d(I) and load) or (qi(I) and not load);
  mxcy: muxcy    port map (S => s(I), DI => '0', CI => ci(I), O => ci(I+1));
  xocy: xorcy       port map ( LI => s(I), CI => ci(I), O => dq(I));
  ff: fdrse    port map ( C => clk, D => dq(I), CE => ec, S => set, R => reset, Q => qi(I));
end generate G1;
end my_arch;
-------------------------------------------------------------------------------------
-- this is a N bit wide LUT shift registers

library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;

entity srle_N is 
generic (N: positive := 16);
    port (
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        a: in STD_LOGIC_VECTOR (3 downto 0);
        ce: in STD_LOGIC;
        clk: in STD_LOGIC;
        q: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end srle_N;

architecture srle_N_arch of srle_N is
begin
g1: for i in d'range generate
  srle: srl16e port map(
                             d => d(i),
                             ce => ce,
                             clk => clk,
                             a0 => a(0),
                             a1 => a(1),
                             a2 => a(2),
                             a3 => a(3),
                             q  => q(i));
end generate;
end srle_N_arch;
---------------------------------------------------------------------------------------------------
-- this is a N bit fifo constructed from LUT shift registers
-- it can hold up to 2^(cntr_length)-1 words
-- when "write" is high, "din" is written to fifo.
-- when "re" is high, fifo has at least one data word in it.
-- when "read" is high, valid data appears one clock later at dout
-- even if "re" goes low after a "read" signal, dout remains valid.
-- when cntr_lenght is set to 2, full goes one when there words are stored in the fifo
-- when cntr_lenght is set to 3, full goes one when five words are stored in the fifo
-- when cntr_lenght is set to 4, full goes one when nine words are stored in the fifo

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library UNISIM;
use UNISIM.VComponents.all;

entity srl_fifo is 
generic (N: integer := 16;
         full_cnt: integer := 11);
    port (
        rst: 	in STD_LOGIC;
        din: 	in STD_LOGIC_VECTOR (N-1 downto 0);
        write: in STD_LOGIC;
        clk: 	in STD_LOGIC;
        read: 	in STD_LOGIC;
        dout: 	out STD_LOGIC_VECTOR (N-1 downto 0);
        wc: 	out STD_LOGIC_VECTOR (3 downto 0);
        full:	out std_logic;
        re: out STD_LOGIC
    );
end srl_fifo;

architecture srl_fifo_arch of srl_fifo is
signal dout_i:			std_logic_vector(N-1 downto 0);
signal count:			std_logic_vector(3 downto 0);
signal full_count:	std_logic_vector(4 downto 0);
signal srl_re:			std_logic; 
signal empty_i:		std_logic; 
signal read_srl:		std_logic;
signal di,s,dq:		std_logic_vector(3 downto 0);
signal ci:				std_logic_vector(4 downto 0);
begin
wc <= count;
full_count <= CONV_STD_LOGIC_VECTOR(full_cnt-3,5);
g1: for i in din'range generate
  srle: srl16e port map(
                             d => din(i),
                             ce => write,
                             clk => clk,
                             a0 => count(0),
                             a1 => count(1),
                             a2 => count(2),
                             a3 => count(3),
                             q  => dout_i(i));
end generate;
re <= srl_re;
read_srl <= read and srl_re;
ci(0) <= write;
G2: for I in count'range generate
  s(I) <= (read and srl_re) xor count(I);
  mxcy: muxcy    port map (S => s(I), DI => count(I), CI => ci(I), O => ci(I+1));
  xocy: xorcy       port map ( LI => s(I), CI => ci(I), O => dq(I));
end generate G2;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(rst = '1')then
	   count <= (others => '1');
	 else
	   count <= dq;
	 end if;
  end if;
end process; 
process(clk)
begin
  if(clk'event and clk = '1')then
    if(rst = '1')then
	   srl_re <= '0';
    elsif(write = '1')then
	   srl_re <= '1';
	 elsif(read = '1' and count = "0000")then
      srl_re <= '0';
 	 end if;
    if(read_srl = '1')then
      dout <= dout_i;
    end if;
	 if(rst = '1')then
		full <= '0';
	 elsif(count = full_count(3 downto 0))then
      full <= write;
	 end if;
  end if;
end process; 
end srl_fifo_arch;
---------------------------------------------------------------------------------------------------
-- LUT FIFO REGISTERED:
---------------------------------------------------------------------------------------------------
-- this is a N bit fifo constructed from LUT shift registers
-- it can hold up to 2^(cntr_length)-1 words
-- it takes two clocks to drop through the fifo when fifo is empty
-- when "write" is high, "din" is written to fifo.
-- when "empty" is low, dout has valid data.
-- when "read" is high, the next valid data appears one clock later at dout if
-- there are still any in the fifo. Otherwise, the last valid data stays until
-- two clocks after the next write.
-- "read" has no effect when "empty" is high
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library UNISIM;
use UNISIM.VComponents.all;

entity srl_fiforeg is 
generic (N: integer := 16;
         full_cnt: integer := 11);
    port (
        rst: 	in STD_LOGIC;
        din: 	in STD_LOGIC_VECTOR (N-1 downto 0);
        write: in STD_LOGIC;
        clk: 	in STD_LOGIC;
        read: 	in STD_LOGIC;
        dout: 	out STD_LOGIC_VECTOR (N-1 downto 0);
        wc: 	out STD_LOGIC_VECTOR (3 downto 0);
        full:	out std_logic;
        empty: out STD_LOGIC
    );
end srl_fiforeg;

architecture srl_fiforeg_arch of srl_fiforeg is
signal dout_i:			std_logic_vector(N-1 downto 0);
signal count:			std_logic_vector(3 downto 0);
signal full_count:	std_logic_vector(4 downto 0);
signal srl_re:			std_logic; 
signal empty_i:		std_logic; 
signal read_srl:		std_logic;
signal di,s,dq:		std_logic_vector(3 downto 0);
signal ci:				std_logic_vector(4 downto 0);
begin
wc <= count;
full_count <= CONV_STD_LOGIC_VECTOR(full_cnt-3,5);
g1: for i in din'range generate
  srle: srl16e port map(
                             d => din(i),
                             ce => write,
                             clk => clk,
                             a0 => count(0),
                             a1 => count(1),
                             a2 => count(2),
                             a3 => count(3),
                             q  => dout_i(i));
end generate;
empty <= empty_i;
read_srl <= (empty_i or read) and srl_re;
ci(0) <= write;
G2: for I in count'range generate
  s(I) <= ((empty_i or read) and srl_re) xor count(I);
  mxcy: muxcy    port map (S => s(I), DI => count(I), CI => ci(I), O => ci(I+1));
  xocy: xorcy       port map ( LI => s(I), CI => ci(I), O => dq(I));
end generate G2;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(rst = '1')then
	   count <= (others => '1');
	 else
	   count <= dq;
	 end if;
  end if;
end process; 
process(clk)
begin
  if(clk'event and clk = '1')then
    if(rst = '1')then
	   srl_re <= '0';
    elsif(write = '1')then
	   srl_re <= '1';
	 elsif(read_srl = '1' and count = "0000")then
      srl_re <= '0';
 	 end if;
    if(rst= '1')then
	   empty_i <= '1';
	 else
	   empty_i <= not srl_re and (empty_i or read);
	 end if;
    if(read_srl = '1')then
      dout <= dout_i;
    end if;
	 if(rst = '1')then
		full <= '0';
	 elsif(count = full_count(3 downto 0))then
      full <= write;
	 end if;
  end if;
end process; 
end srl_fiforeg_arch;
---------------------------------------------------------------------------------------------------
-- ADD one N:
---------------------------------------------------------------------------------------------------
-- this adder implements a carry chain.

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library UNISIM;
use UNISIM.VComponents.all;
entity add_oneN is
    generic ( N: integer := 24);
    port (
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        rst: in STD_LOGIC;
        sum: out STD_LOGIC_VECTOR (N-1 downto 0);
		  carry: out STD_LOGIC
    );
end add_oneN;
architecture add_oneN_arch of add_oneN is
signal	ci:	std_logic_vector(N downto 0);
signal	di,s:	std_logic_vector(N-1 downto 0);

begin
ci(0) <= '1';
carry <= ci(N);
G1: for I in 0 to N-1 generate
  s(I) <= rst or d(I);
  mxcy: muxcy    port map (S => s(I),
                           DI => '0',
                           CI => ci(I),
                           O => ci(I+1));
  xocy: xorcy    port map (LI => s(I),
  		           CI => ci(I),
  		           O => sum(I));
end generate G1;     

end add_oneN_arch;
---------------------------------------------------------------------------------------------------
-- this is a n-bit window comparator for virtex2.
-- This is implemeting a carry chain.

library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;

entity compWN is
    generic (N : integer := 12;
	          ul_include : std_logic := '0';
				 ll_include : std_logic := '1');
    port (
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        u_limit: in STD_LOGIC_VECTOR (N-1 downto 0);
        l_limit: in STD_LOGIC_VECTOR (N-1 downto 0);
        n_good: out STD_LOGIC
    );
end compWN;

architecture my_arch of compWN is
signal	s_u:	std_logic_vector(N-1 downto 0);
signal	c_u:	std_logic_vector(N downto 0);
signal	s_l:	std_logic_vector(N-1 downto 0);
signal	c_l:	std_logic_vector(N downto 0);
signal   u_bad: std_logic;
begin
c_u(0) <= not ul_include;
c_l(0) <= not ll_include;
G1: for j in d'range generate
  s_u(j) <= not d(j) xor u_limit(j);
  mxcy: muxcy    port map (S => s_u(j), DI => d(j), CI => c_u(j), O => c_u(j+1));
end generate G1;
G2: for j in d'range generate
  s_l(j) <= not d(j) xor l_limit(j);
  mxcy: muxcy    port map (S => s_l(j), DI => l_limit(j), CI => c_l(j), O => c_l(j+1));
end generate G2;
i_orcy_u: orcy port map(i => '0', ci => c_u(N), o => u_bad);
i_orcy_l: orcy port map(i => u_bad, ci => c_l(N), o => n_good);
end my_arch;
-------------------------------------------------------------------------------------
-- this is a N-bit adder/subtractor
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;
entity adsuN is
generic (N: positive := 8);
  port (
    a: in std_logic_vector (N-1 downto 0);
    b: in std_logic_vector (N-1 downto 0);
    add: in std_logic;
    ci: in std_logic;
    s: out std_logic_vector (N-1 downto 0);
    co: out std_logic;
    ovfl: out std_logic
  );
end adsuN;

architecture my_arch of adsuN is
signal	i,o:	std_logic_vector(N-1 downto 0);
signal	c:	std_logic_vector(N downto 0);
begin
co <= c(N);
ovfl <= c(N) xor o(N-2);
c(0) <= ci;
G1: for j in a'range generate
  i(j) <= not add xor a(j) xor b(j);
  mxcy: muxcy_d    port map (S => i(j), DI => a(j), CI => c(j), LO => c(j+1), O => o(j));
  xocy: xorcy       port map ( LI => i(j), CI => c(j), O => s(j));
end generate G1;
end my_arch;
------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library UNISIM;
use UNISIM.VComponents.all;

entity updown_cntr is 
generic (N: integer := 4);
    port (
        rst: in STD_LOGIC;
        set: in STD_LOGIC;
        ce: in STD_LOGIC;
        up: in STD_LOGIC;
        down: in STD_LOGIC;
        clk: in STD_LOGIC;
        q: out STD_LOGIC_VECTOR (N-1 downto 0);
        carry: out STD_LOGIC
    );
end updown_cntr;

architecture updown_cntr_arch of updown_cntr is
signal q_i:				std_logic_vector(N-1 downto 0);
signal di,s,dq:		std_logic_vector(N-1 downto 0);
signal ci:				std_logic_vector(N downto 0);
begin
q <= q_i;
carry <= ci(N);
ci(0) <= up;
G1: for I in q_i'range generate
  s(I) <= down xor q_i(I);
  mxcy: muxcy    port map (S => s(I), DI => q_i(I), CI => ci(I), O => ci(I+1));
  xocy: xorcy       port map ( LI => s(I), CI => ci(I), O => dq(I));
  ff: fdrse    port map ( C => clk, D => dq(I), CE => ce, r => rst, S => set, Q => q_i(I));
end generate G1;
end updown_cntr_arch;
-------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;
entity ram16XNd is
    generic ( N: positive := 16);
    port (
        clk: in STD_LOGIC;
        we: in STD_LOGIC;
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        a: in STD_LOGIC_VECTOR (3 downto 0);
        dpra: in STD_LOGIC_VECTOR (3 downto 0);
        spo: out STD_LOGIC_VECTOR (N-1 downto 0);
        dpo: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end ram16XNd;
architecture my_arch of ram16XNd is
begin
g0: for i in d'range generate
ram: ram16x1d
    port map(
             we => we,
             wclk => clk,
             d => d(i),
             a0 => a(0),
             a1 => a(1),
             a2 => a(2),
             a3 => a(3),
             dpra0 => dpra(0),
             dpra1 => dpra(1),
             dpra2 => dpra(2),
             dpra3 => dpra(3),
             spo => spo(i),
             dpo => dpo(i));
end generate;
end my_arch;
---------------------------------------------------------------------------------------------------
-- this is a n-bit comparator.
-- This is implemeting a carry chain.

library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;

entity compCN is
-- N must be even
    generic (N : integer := 36);
    port (
        a: in STD_LOGIC_VECTOR (N-1 downto 0);
        b: in STD_LOGIC_VECTOR (N-1 downto 0);
		  en: in STD_LOGIC;
        equal: out STD_LOGIC
    );
end compCN;

architecture my_arch of compCN is
signal	s:	std_logic_vector((N-1)/2 downto 0);
signal	ci:	std_logic_vector(N/2 downto 0);
begin
ci(0) <= en;
equal <= ci(N/2);
G1: for j in 0 to (N/2)-1 generate
  s(j) <= (a(2*j) xnor b(2*j)) and (a(2*j+1) xnor b(2*j+1));
  mxcy: muxcy    port map (S => s(j), DI => '0', CI => ci(j), O => ci(j+1));
end generate G1;
end my_arch;
-------------------------------------------------------------------------------------
-- this is a loadable down counter with set, reset and clock enable inputs 
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;
entity l_d_cntr is
generic (N: positive := 8; ld_minus_one : boolean := false );
  port (
    d: in std_logic_vector (N-1 downto 0);
-- load = '1' and ec = '1' loads d into q
    load: in std_logic;
-- reset resets the counter to all zeros, reset overrides set and ec
    reset: in std_logic;
-- set will set the counter to all ones
    set: in std_logic;
-- ec = '1' is needed to load or count 
    ec: in std_logic;
-- en_cnt = '1' is needed to count. If load with en_cnt = '1',
-- d - 1 is loaded. 
    en_cnt: in std_logic;
-- clock input
    clk: in std_logic;
-- q is the counter output
    q: out std_logic_vector (N-1 downto 0);
-- carry out
    carry: out std_logic
  );
end l_d_cntr;

architecture my_arch of l_d_cntr is
signal	qi,s,dq:	std_logic_vector(N-1 downto 0);
signal	ci:	std_logic_vector(N downto 0);
begin
q <= qi;
carry <= ci(N);
ci(0) <= not en_cnt and not load when ld_minus_one else not en_cnt or load;
G1: for I in q'range generate
  s(I) <= (not d(I) and load) or (not qi(I) and not load);
  mxcy: muxcy    port map (S => s(I), DI => '1', CI => ci(I), O => ci(I+1));
  xocy: xorcy       port map ( LI => s(I), CI => ci(I), O => dq(I));
  ff: fdrse    port map ( C => clk, D => dq(I), CE => ec, S => set, R => reset, Q => qi(I));
end generate G1;
end my_arch;
---------------------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;
entity ram16XNs is
    generic ( N: positive := 16);
    port (
        clk: in STD_LOGIC;
        we: in STD_LOGIC;
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        a: in STD_LOGIC_VECTOR (3 downto 0);
        o: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end ram16XNs;

architecture ram16XNs_arch of ram16XNs is
begin
g0: for i in d'range generate
ram: ram16x1s
    port map(
             we => we,
             wclk => clk,
             d => d(i),
             a0 => a(0),
             a1 => a(1),
             a2 => a(2),
             a3 => a(3),
             o => o(i));
end generate;
end ram16XNs_arch; 
-------------------------------------------------------------------------------------
-- this is a N-bit adder/subtractor
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;
entity addN is
generic (N: positive := 8);
  port (
    a: in std_logic_vector (N-1 downto 0);
    b: in std_logic_vector (N-1 downto 0);
    rst_n: in std_logic;
    ci: in std_logic;
    s: out std_logic_vector (N-1 downto 0);
    co: out std_logic;
    ovfl: out std_logic
  );
end addN;

architecture my_arch of addN is
signal	i,o,di:	std_logic_vector(N-1 downto 0);
signal	c:	std_logic_vector(N downto 0);
begin
co <= c(N);
ovfl <= c(N) xor o(N-2);
c(0) <= ci;
G1: for j in a'range generate
  i(j) <= rst_n and (a(j) xor b(j));
  mxcy: muxcy_d    port map (S => i(j), DI => di(j), CI => c(j), LO => c(j+1), O => o(j));
  multand: mult_and       port map ( I0 => b(j), I1 => rst_n, LO => di(j));
  xocy: xorcy       port map ( LI => i(j), CI => c(j), O => s(j));
end generate G1;
end my_arch;
---------------------------------------------------------------------------------------------------

