library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
package vtx_pack is
-------------------------------------------------------------------------------------
-- this is a N bit wide LUT shift registers
component srle_N
generic (N: positive := 16);
    port (
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        a: in STD_LOGIC_VECTOR (3 downto 0);
        ce: in STD_LOGIC;
        clk: in STD_LOGIC;
        q: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end component;
-------------------------------------------------------------------------------------
-- delay d from 2 to 17 clocks with ce = '1'
component delayN
    generic (N: positive := 2;
	          INIT : bit_vector := x"0000");
    port (
        c: in STD_LOGIC;
        d: in STD_LOGIC;
        ce: in STD_LOGIC;
        q: out STD_LOGIC
    );
end component;
-------------------------------------------------------------------------------------
component adsuN
generic (N: positive := 8);
  port (
    a: in std_logic_vector (N-1 downto 0);
    b: in std_logic_vector (N-1 downto 0);
    add: in std_logic;
    ci: in std_logic;
    s: out std_logic_vector (N-1 downto 0);
    co: out std_logic;
    ovfl: out std_logic
  );
end component;
-------------------------------------------------------------------------------------
component addN
generic (N: positive := 8);
  port (
    a: in std_logic_vector (N-1 downto 0);
    b: in std_logic_vector (N-1 downto 0);
    rst_n: in std_logic;
    ci: in std_logic;
    s: out std_logic_vector (N-1 downto 0);
    co: out std_logic;
    ovfl: out std_logic
  );
end component;
-------------------------------------------------------------------------------------
-- this is a loadable up counter with set, reset and clock enable inputs 
component l_cntr
generic (N: positive := 8; ld_plus_one : boolean := false );
  port (
    d: in std_logic_vector (N-1 downto 0);
    load: in std_logic;
    reset: in std_logic;
    set: in std_logic;
    ec: in std_logic;
    en_cnt: in std_logic;
    clk: in std_logic;
    q: out std_logic_vector (N-1 downto 0);
    carry: out std_logic
  );
end component;
-------------------------------------------------------------------------------------
-- this is a loadable up counter with set, reset and clock enable inputs 
component l_d_cntr
generic (N: positive := 8; ld_minus_one : boolean := false );
  port (
    d: in std_logic_vector (N-1 downto 0);
    load: in std_logic;
    reset: in std_logic;
    set: in std_logic;
    ec: in std_logic;
    en_cnt: in std_logic;
    clk: in std_logic;
    q: out std_logic_vector (N-1 downto 0);
    carry: out std_logic
  );
end component;
----------------------------------------------------------------------------------------
-- declare lut dpram
component ram16XNd
    generic ( N: positive := 16);
    port (
        clk: in STD_LOGIC;
        we: in STD_LOGIC;
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        a: in STD_LOGIC_VECTOR (3 downto 0);
        dpra: in STD_LOGIC_VECTOR (3 downto 0);
        spo: out STD_LOGIC_VECTOR (N-1 downto 0);
        dpo: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end component;
----------------------------------------------------------------------------------------
component ram16XNs
    generic (N: positive := 16);
    port (
        clk: in STD_LOGIC;
        we: in STD_LOGIC;
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        a: in STD_LOGIC_VECTOR (3 downto 0);
        o: out STD_LOGIC_VECTOR (N-1 downto 0));
end component;
-------------------------------------------------------------------------------------
-- window comparator. default less than upper limit and greater eqaul than lower limit
component compWN
    generic (N : integer := 12;
	          ul_include : std_logic := '0';
				 ll_include : std_logic := '1');
    port (
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        u_limit: in STD_LOGIC_VECTOR (N-1 downto 0);
        l_limit: in STD_LOGIC_VECTOR (N-1 downto 0);
        n_good: out STD_LOGIC
    );
end component;
--------------------------------------------------------------------------------------
component srl_fifo 
generic (N: integer := 16; full_cnt: integer := 11);
    port (
        rst: in STD_LOGIC;
        din: in STD_LOGIC_VECTOR (N-1 downto 0);
        write: in STD_LOGIC;
        clk: in STD_LOGIC;
        read: in STD_LOGIC;
        dout: out STD_LOGIC_VECTOR (N-1 downto 0);
        wc: 	out STD_LOGIC_VECTOR (3 downto 0);
        full:	out std_logic;
        re: out STD_LOGIC
    );
end component;
---------------------------------------------------------------------------------------
component srl_fiforeg is 
generic (N: integer := 16; full_cnt: integer := 11);
    port (
        rst: 	in STD_LOGIC;
        din: 	in STD_LOGIC_VECTOR (N-1 downto 0);
        write: 	in STD_LOGIC;
        clk: 	in STD_LOGIC;
        read: 	in STD_LOGIC;
        dout: 	out STD_LOGIC_VECTOR (N-1 downto 0);
        wc: 	out STD_LOGIC_VECTOR (3 downto 0);
        full:	out std_logic;
        empty: 	out STD_LOGIC);
end component;
-----------------------------------------------------------------------
component add_oneN 
    generic ( N: integer := 24);
    port (
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        rst: in STD_LOGIC;
        sum: out STD_LOGIC_VECTOR (N-1 downto 0);
		  carry: out STD_LOGIC);
end component;
---------------------------------------------------------------------------------
component delayt_N
  generic ( T: positive := 2; -- Minimum value!
   	    N: positive := 16);
    port (
        clk: in STD_LOGIC;
        d: in STD_LOGIC_vector(N-1 downto 0);
        ce: in STD_LOGIC;
        q: out STD_LOGIC_vector(N-1 downto 0)
         );
end component;
---------------------------------------------------------------------------------
COMPONENT updown_cntr 
generic (N: integer := 4);
    port (
        rst: in STD_LOGIC;
        set: in STD_LOGIC;
        ce: in STD_LOGIC;
        up: in STD_LOGIC;
        down: in STD_LOGIC;
        clk: in STD_LOGIC;
        q: out STD_LOGIC_VECTOR (N-1 downto 0);
        carry: out STD_LOGIC
    );
END COMPONENT;
---------------------------------------------------------------------------------
component ddr_io
    port (
        clk: in STD_LOGIC;
        clkn: in STD_LOGIC;
        pad: inout std_logic;
        t: in STD_LOGIC;
        d0: in STD_LOGIC;
        d1: in STD_LOGIC;
        iclk: in STD_LOGIC;
        iclkn: in STD_LOGIC;
        q0: out std_logic; -- rising edge       
        q1: out std_logic  -- falling edge    
    );
end component;
----------------------------------------------------------------------
component compCN
-- N must be even
    generic (N : integer := 36);
    port (
        a: in STD_LOGIC_VECTOR (N-1 downto 0);
        b: in STD_LOGIC_VECTOR (N-1 downto 0);
		  en: in STD_LOGIC;
        equal: out STD_LOGIC
    );
end component;
end vtx_pack;
package body vtx_pack is
end vtx_pack;
